Imports System.Collections.Generic
Imports System.Drawing.Drawing2D
Imports System.Globalization
Imports System.IO
Imports System.Windows.Forms

Class Form1

    ' This array stores the latitude and longitude of Nebraska in the United States
    Private geographicCoordinates() As PointF
    ' This map projection flattens 3D coordinates to 2D coordinates.
    Private plateCaree As New MapProjection()
    ' This array stores the same coordinates but as projected coordinates
    Private projectedCoordinates() As PointF
    ' This rectangle represents the "viewport", the portion of the coordinates we want to see.
    ' The viewport defaults to a rectangle which fits the entire state of Nebraska.
    Private viewport As RectangleF = RectangleF.FromLTRB(-11583105.0F, -4787050.0F, -10609638.0F, -4452767.5F)
    ' Use a matrix for translation and scaling
    Private transform As New Matrix()
    ' This brush and pen are used to paint Nebraska
    Private brush As New SolidBrush(Color.Green)
    Private pen As New Pen(Color.Black, 0.001F)

    Sub New()

        ' This call is required by the Windows Form Designer.
        InitializeComponent()

        ' Redraw the control when it's resized
        SetStyle(ControlStyles.ResizeRedraw, True)

        ' This example displays the state of Nebraska on-screen.  Geographic coordinates
        ' which form Nebraska are stored in a text file, Nebraska.txt.
        LoadData()

        ' For this example, we will "flatten" the state of ______ using the Plate Caree
        ' map projection.  Once this is done, we can draw these coordinates onto the form.

        ' First, create a new map projection
        Dim plateCaree As New MapProjection()

        ' Next, make an array to store our projected coordinates.
        ReDim projectedCoordinates(geographicCoordinates.Length - 1)

        ' For each geographic coordinate, project it
        Dim index As Integer
        For index = 0 To geographicCoordinates.Length - 1
            ' Convert the geographic coordinate to a projected coordinate
            projectedCoordinates(index) = plateCaree.Project(geographicCoordinates(index))
        Next

    End Sub

    Protected Overrides Sub OnPaint(ByVal e As PaintEventArgs)

        e.Graphics.Clear(Color.White)

        ' In this example, the entire form surface is painted.  Since coordinates have been
        ' projected, the next task is to scale and translate our "viewport" to the form's own dimensions


        ' If the shape of the Form is different from the shape of the viewport, the reulting 
        ' shape will become distorted.  To prevent this, we can make the aspect ratio of the
        ' viewport match the form's aspect ratio.

        ' Calculate the aspect ratio of the Form
        Dim pixelAspectRatio As Single = Me.Width / Me.Height

        ' Calculate the aspect ratio of the viewport
        Dim projectedAspectRatio As Single = viewport.Width / viewport.Height

        ' Make a copy of the viewport that we can change
        Dim adjustedViewport As RectangleF = viewport

        ' Is the Form's aspect ratio larger than the viewport's ratio?
        If pixelAspectRatio > projectedAspectRatio Then
            ' Yes.  Increase the width of the viewport
            adjustedViewport.Inflate( _
                (pixelAspectRatio * adjustedViewport.Height - adjustedViewport.Width) / 2, _
                0)
            ' Is the viewport's aspect ratio larger than the form's ratio?
        ElseIf pixelAspectRatio < projectedAspectRatio Then
            ' Yes.  Increase the height of the viewport
            adjustedViewport.Inflate( _
                0, _
                (adjustedViewport.Width / pixelAspectRatio - adjustedViewport.Height) / 2)
        End If

        ' Reset the transform
        transform.Reset()

        ' First, translate all projected points so that they match up with pixel 0,0
        transform.Translate(-adjustedViewport.X, adjustedViewport.Y, MatrixOrder.Append)

        ' Next, scale all points so that the viewport fits inside the form.
        transform.Scale(Me.Width / adjustedViewport.Width, _
                       Me.Height / -adjustedViewport.Height, MatrixOrder.Append)

        ' Apply this transform to all graphics operations
        e.Graphics.Transform = transform

        ' Now draw nebraska using a green interior and black outline
        e.Graphics.FillPolygon(brush, projectedCoordinates)
        e.Graphics.DrawPolygon(pen, projectedCoordinates)

    End Sub

    Public Sub ZoomIn()

        ' First, get the width of the viewport, then calculate 10% of it
        Dim zoomWidthAmount As Single = -viewport.Width * 0.1F
        Dim zoomHeightAmount As String = -viewport.Height * 0.1F

        ' Next, apply the amounts to shrink (yes, shrink) the viewport to zoom in
        viewport.Inflate(zoomWidthAmount, zoomHeightAmount)

        ' And repaint
        Refresh()
    End Sub

    Public Sub ZoomOut()

        ' First, get the width of the viewport, then calculate 10% of it
        Dim zoomWidthAmount As Single = viewport.Width * 0.1F
        Dim zoomHeightAmount As String = viewport.Height * 0.1F

        ' Next, apply the amounts to shrink (yes, shrink) the viewport to zoom in
        viewport.Inflate(zoomWidthAmount, zoomHeightAmount)

        ' And repaint
        Refresh()
    End Sub

    ' Reads longitude and latitude values from a text file
    Private Sub LoadData()

        ' This example stores latitude and longitude coordinates in the file Nebraska.txt.
        ' Read in each line and turn it into a PointF object.
        Dim reader As StreamReader = File.OpenText("..\..\Nebraska.txt")

        ' Make a list to store the coordinates
        Dim coordinates As New List(Of PointF)

        ' Read everything in
        While Not reader.EndOfStream

            ' Each line is in the form "latitude, longitude".  Parse each into a float, then
            ' create a PointF object from it.
            Dim values() As String = reader.ReadLine().Split(",")

            ' Parse both values into a PointF object
            coordinates.Add( _
                New PointF( _
                    Single.Parse(values(1), CultureInfo.InvariantCulture), _
                    Single.Parse(values(0), CultureInfo.InvariantCulture)))
        End While

        ' Close the reader
        reader.Close()

        ' Finally, make an array of projected coordinates
        geographicCoordinates = coordinates.ToArray()
    End Sub

    Public Sub PanUp()
        ' First, get the height of the viewport, then calculate 10% of it
        Dim zoomHeightAmount As Single = -viewport.Height * 0.1F

        ' Shift the viewport by this amount
        viewport.Offset(0.0F, zoomHeightAmount)

        ' Finally, redraw the form
        Invalidate()
    End Sub

    Public Sub PanDown()
        ' First, get the height of the viewport, then calculate 10% of it
        Dim zoomHeightAmount As Single = viewport.Height * 0.1F

        ' Shift the viewport by this amount
        viewport.Offset(0.0F, zoomHeightAmount)

        ' Finally, redraw the form
        Invalidate()
    End Sub

    Protected Overrides Sub OnKeyDown(ByVal e As KeyEventArgs)

        Select Case e.KeyValue
            Case 38
                PanUp()
            Case 40
                PanDown()
        End Select

    End Sub

    Protected Overrides Sub OnMouseWheel(ByVal e As MouseEventArgs)

        If e.Delta < 0 Then
            ZoomOut()
        Else
            ZoomIn()
        End If

    End Sub

    Protected Overrides Sub OnMouseMove(ByVal e As MouseEventArgs)

        ' When the mouse is moved over the map, show the coordinate the mouse
        ' is hovering over, in all three coordinate systems: pixel, projected, and geographic.

        ' We use a Matrix to convert from projected coordinates to pixel coordinates.
        ' The *inverse* of this matrix is used to convert pixels back into projected coordinates.

        ' First, make a copy of the current transform
        Dim reverseTransform As Matrix = transform.Clone()

        ' Next, invert the transform
        reverseTransform.Invert()

        ' We can now use this to calculate a projected coordinate.
        Dim projectedCoordinate() As PointF
        ReDim projectedCoordinate(1)
        projectedCoordinate(0) = e.Location
        reverseTransform.TransformPoints(projectedCoordinate)

        ' "Points" now contains projected coordinates.  Use our projection to convert
        ' this into geographic (latitude/longitude) coordinates.
        Dim geographicCoordinate As PointF = plateCaree.Deproject(projectedCoordinate(0))

        ' Finally, display all three coordinate values: pixel, projected, geographic
        pixelCoordinateLabel.Text = "Pixel: " + e.Location.ToString()
        projectedCoordinateLabel.Text = "Projected: " + projectedCoordinate(0).ToString()
        geographicCoordinateLabel.Text = "Geographic: " + geographicCoordinate.ToString()

    End Sub
    Protected Overrides Sub OnClosed(ByVal e As System.EventArgs)
        MyBase.OnClosed(e)

        ' Clean up our unmanaged resources. 
        ' These will get disposed of when the app ends, but this would be essential
        ' if this were not the only form.
        transform.Dispose()
        brush.Dispose()
        pen.Dispose()

    End Sub
End Class
